@@ -12,6 +12,8 @@
 # Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
 # Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
 # Copyright 2017 Nicolas Agustín Torres <nicolastrres@gmail.com>              #
+# Copyright 2018 Jess Morgan <979404+JessMorgan@users.noreply.github.com>      #
+# Copyright 2018 per1234 <accounts@perglass.com>                               #
 # Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
 #                                                                              #
 # This file is part of PyGithub.                                               #
@@ -33,9 +35,9 @@
 ################################################################################
 
 import github.GithubObject
-
 import github.NamedUser
 
+import Consts
 
 class PullRequestComment(github.GithubObject.CompletableGithubObject):
     """
@@ -203,7 +205,7 @@ class PullRequestComment(github.GithubObject.CompletableGithubObject):
             self._requester,
             self.url + "/reactions",
             None,
-            headers={'Accept': 'application/vnd.github.squirrel-girl-preview'}
+            headers={'Accept': Consts.mediaTypeReactionsPreview}
         )
 
     def create_reaction(self, reaction_type):
@@ -224,7 +226,7 @@ class PullRequestComment(github.GithubObject.CompletableGithubObject):
             "POST",
             self.url + "/reactions",
             input=post_parameters,
-            headers={'Accept': 'application/vnd.github.squirrel-girl-preview'}
+            headers={'Accept': Consts.mediaTypeReactionsPreview}
         )
         return github.Reaction.Reaction(self._requester, headers, data, completed=True)
 
